;**** I 2 C _ E E P _ 2 4 C 6 4 . A S M ************************************
;*
;* Title		: I2C - EEPROM Implementation for 24C32 and 24C64
;* Version		: 1.0 (BETA)
;* Last updated		: 04.01.2003
;* Target		: AT90Sxxxx (any AVR device)
;*
;* DESCRIPTION
;* 	Basic routines for communicating with a I2C - EEPROM. This
;*	implementation is limited to one I2C - EEPROM.
;*
;* STATISTICS
;*	Code Size	: 86 words
;*	Register Usage	: 3 High, 0 Low
;*
;***************************************************************************

.EQU I2C_EEP_DEV_ADR	= 0xA0	; IC device address of the EEPROM

;**** Global Register Variables ****
.DEF i2c_eep_data	= r21
.DEF i2c_eep_adr_l	= r22
.DEF i2c_eep_adr_h	= r23

;***************************************************************************
;*
;* FUNCTION
;*	i2c_eep_write
;*
;* DESCRIPTION
;*	Writes data (one byte) to the EEPROM and waits until the data is
;*	programmed.
;*
;* USAGE
;*	i2c_eep_data   - Contains data to be transmitted.
;*	i2c_eep_adr_l  - Contains the low byte of the address in the EEPROM
;*	i2c_eep_adr_h  - Contains the high byte of the address in the EEPROM
;*
;* RETURN
;*	T-flag - Clear if the slave respond with an Acknowledge.
;*
;* REGISTERS USED
;*	none
;*
;***************************************************************************
i2c_eep_write:
	rcall	i2c_start

	ldi	i2c_data, (I2C_EEP_DEV_ADR | I2C_W)
	rcall	i2c_write
	brts	i2c_eep_write_error	; if no Acknowledge received => error

	mov	i2c_data, i2c_eep_adr_h
	rcall	i2c_write
	brts	i2c_eep_write_error	; if no Acknowledge received => error

	mov	i2c_data, i2c_eep_adr_l
	rcall	i2c_write
	brts	i2c_eep_write_error	; if no Acknowledge received => error

	mov	i2c_data, i2c_eep_data
	rcall	i2c_write
	
	rcall	i2c_stop

i2c_eep_write_wait:
	rcall	i2c_start

	ldi	i2c_data, (I2C_EEP_DEV_ADR | I2C_W)
	rcall	i2c_write

	rcall	i2c_stop
	brts	i2c_eep_write_wait	; if no Acknowledge received => EEPROM is programming

	ret

i2c_eep_write_error:
	rcall	i2c_stop

	ret

;***************************************************************************
;*
;* FUNCTION
;*	i2c_eep_read
;*
;* DESCRIPTION
;*	Reads data (one byte) to the EEPROM.
;*
;* USAGE
;*	i2c_eep_data   - Contains received data.
;*	i2c_eep_adr_l  - Contains the low byte of the address in the EEPROM
;*	i2c_eep_adr_h  - Contains the high byte of the address in the EEPROM
;*
;* RETURN
;*	T-flag - Clear if the slave respond with an Acknowledge.
;*
;* REGISTERS USED
;*	none
;*
;***************************************************************************
i2c_eep_read:
	rcall	i2c_start

	ldi	i2c_data, (I2C_EEP_DEV_ADR | I2C_W)
	rcall	i2c_write
	brts	i2c_eep_read_error	; if no Acknowledge received => error

	mov	i2c_data, i2c_eep_adr_h
	rcall	i2c_write
	brts	i2c_eep_write_error	; if no Acknowledge received => error

	mov	i2c_data, i2c_eep_adr_l
	rcall	i2c_write
	brts	i2c_eep_read_error	; if no Acknowledge received => error
	
	rcall	i2c_stop
	rcall	i2c_start
	
	ldi	i2c_data, (I2C_EEP_DEV_ADR | I2C_R)
	rcall	i2c_write
	brts	i2c_eep_read_error	; if no Acknowledge received => error
	
	rcall	i2c_read
	brts	i2c_eep_read_error	; if no Acknowledge received => error
	mov	i2c_eep_data, i2c_data

i2c_eep_read_error:
	rcall	i2c_stop

	ret

;***************************************************************************
;*
;* FUNCTION
;*	i2c_eep_page_write
;*
;* DESCRIPTION
;*	Writes page (max 32 byte) from the SRAM into the EEPROM and waits
;*	until the data is programmed.
;*
;* USAGE
;*	i2c_eep_data   - Contains the data size / lenght
;*	i2c_eep_adr_l  - Contains the low byte of the address in the EEPROM
;*	i2c_eep_adr_h  - Contains the high byte of the address in the EEPROM
;*	Z-Pointer      - points to the start address in the SRAM of the data
;*		         to be transmitted.
;*
;* RETURN
;*	T-flag - Clear if the slave respond with an Acknowledge.
;*
;* REGISTERS USED
;*	none
;*
;***************************************************************************
i2c_eep_page_write:
	rcall	i2c_start

	ldi	i2c_data, (I2C_EEP_DEV_ADR | I2C_W)
	rcall	i2c_write
	brts	i2c_eep_page_write_error	; if no Acknowledge received => error

	mov	i2c_data, i2c_eep_adr_h
	rcall	i2c_write
	brts	i2c_eep_write_error	; if no Acknowledge received => error

	mov	i2c_data, i2c_eep_adr_l
	rcall	i2c_write
	brts	i2c_eep_page_write_error	; if no Acknowledge received => error

i2c_eep_page_write_next:
	ld	i2c_data, Z+
	rcall	i2c_write
	brts	i2c_eep_page_write_error	; if no Acknowledge received => error

	dec	i2c_eep_data
	brne	i2c_eep_page_write_next
	
	rcall	i2c_stop

i2c_eep_page_write_wait:
	rcall	i2c_start

	ldi	i2c_data, (I2C_EEP_DEV_ADR | I2C_W)
	rcall	i2c_write

	rcall	i2c_stop
	brts	i2c_eep_page_write_wait	; if no Acknowledge received => EEPROM is programming

	ret

i2c_eep_page_write_error:
	rcall	i2c_stop

	ret

;***************************************************************************
;*
;* FUNCTION
;*	i2c_eep_page_read
;*
;* DESCRIPTION
;*	Reads page (max 32 byte) from EEPROM into the the SRAM.
;*
;* USAGE
;*	i2c_eep_data   - Contains the data size / lenght
;*	i2c_eep_adr_l  - Contains the low byte of the address in the EEPROM
;*	i2c_eep_adr_h  - Contains the high byte of the address in the EEPROM
;*	Z-Pointer      - points to the start address in the SRAM of the data.
;*
;* RETURN
;*	T-flag - Clear if the slave respond with an Acknowledge.
;*
;* REGISTERS USED
;*	none
;*
;***************************************************************************
i2c_eep_page_read:
	rcall	i2c_start

	ldi	i2c_data, (I2C_EEP_DEV_ADR | I2C_W)
	rcall	i2c_write
	brts	i2c_eep_page_read_error	; if no Acknowledge received => error

	mov	i2c_data, i2c_eep_adr_h
	rcall	i2c_write
	brts	i2c_eep_write_error	; if no Acknowledge received => error

	mov	i2c_data, i2c_eep_adr_l
	rcall	i2c_write
	brts	i2c_eep_page_read_error	; if no Acknowledge received => error
	
	rcall	i2c_stop
	rcall	i2c_start
	
	ldi	i2c_data, (I2C_EEP_DEV_ADR | I2C_R)
	rcall	i2c_write
	brts	i2c_eep_page_read_error	; if no Acknowledge received => error
	
i2c_eep_page_read_next:
	rcall	i2c_read
	brts	i2c_eep_page_read_error	; if no Acknowledge received => error

	st	Z+, i2c_data

	dec	i2c_eep_data
	brne	i2c_eep_page_read_next


i2c_eep_page_read_error:
	rcall	i2c_stop

	ret
